==> multipath-tools-0.3.2.tar.bz2: multipath-tools-0.3.2/multipathd/callout.c <==
#include <stdio.h>
#include <sys/stat.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <stdlib.h>
#include <sys/wait.h>
#include <errno.h>
#include "log.h"

#define PROGRAM_SIZE	100
#define FIELD_PROGRAM

#define strfieldcpy(to, from) \
do { \
	to[sizeof(to)-1] = '\0'; \
	strncpy(to, from, sizeof(to)-1); \
} while (0)

int execute_program(char *path, char *value, int len)
{
	int retval;
	int count;
	int status;
	int fds[2];
	pid_t pid;
	char *pos;
	char arg[PROGRAM_SIZE];
	char *argv[sizeof(arg) / 2];
	int i;

	i = 0;
	if (strchr(path, ' ')) {
		strfieldcpy(arg, path);
		pos = arg;
		while (pos != NULL) {
			if (pos[0] == '\'') {
				/* don't separate if in apostrophes */
				pos++;
				argv[i] = strsep(&pos, "\'");
				while (pos[0] == ' ')
					pos++;
		} else {
				argv[i] = strsep(&pos, " ");
			}
			LOG(3, "arg[%i] '%s'", i, argv[i]);
			i++;
		}
	}
	argv[i] =  NULL;

	retval = pipe(fds);
	if (retval != 0) {
		LOG(3, "pipe failed");
		return -1;
	}


	pid = fork();

	switch(pid) {
	case 0:
		/* child */
		close(STDOUT_FILENO);

		/* dup write side of pipe to STDOUT */
		dup(fds[1]);

		LOG(3, "execute '%s' with given arguments", argv[0]);
		retval = execv(argv[0], argv);

		LOG(3, FIELD_PROGRAM " execution of '%s' failed", path);
		exit(-1);
	case -1:
		LOG(3, "fork failed");
		return -1;
	default:
		/* parent reads from fds[0] */
		close(fds[1]);
		retval = 0;
		i = 0;
		while (1) {
			count = read(fds[0], value + i, len - i-1);
			if (count <= 0)
				break;

			i += count;
			if (i >= len-1) {
				LOG(3, "result len %d too short", len);
				retval = -1;
				break;
			}
		}

		if (count < 0) {
			LOG(3, "read failed with '%s'", strerror(errno));
			retval = -1;
		}

		if (i > 0 && value[i-1] == '\n')
			i--;
		value[i] = '\0';
		LOG(3, "result is '%s'", value);

		close(fds[0]);
		wait(&status);

		if (!WIFEXITED(status) || (WEXITSTATUS(status) != 0)) {
			LOG(3, "exec program status 0x%x", status);
			retval = -1;
		}
	}
	return retval;
}

==> multipath-tools-0.3.4.tar.bz2: multipath-tools-0.3.4/multipathd/callout.c <==
#include <stdio.h>
#include <sys/stat.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <stdlib.h>
#include <sys/wait.h>
#include <errno.h>
#include <syslog.h>

#define PROGRAM_SIZE	100
#define FIELD_PROGRAM

#define strfieldcpy(to, from) \
do { \
	to[sizeof(to)-1] = '\0'; \
	strncpy(to, from, sizeof(to)-1); \
} while (0)

int execute_program(char *path, char *value, int len)
{
	int retval;
	int count;
	int status;
	int fds[2];
	pid_t pid;
	char *pos;
	char arg[PROGRAM_SIZE];
	char *argv[sizeof(arg) / 2];
	int i;

	i = 0;
	if (strchr(path, ' ')) {
		strfieldcpy(arg, path);
		pos = arg;
		while (pos != NULL) {
			if (pos[0] == '\'') {
				/* don't separate if in apostrophes */
				pos++;
				argv[i] = strsep(&pos, "\'");
				while (pos[0] == ' ')
					pos++;
		} else {
				argv[i] = strsep(&pos, " ");
			}
			syslog(LOG_DEBUG, "arg[%i] '%s'", i, argv[i]);
			i++;
		}
	}
	argv[i] =  NULL;

	retval = pipe(fds);
	if (retval != 0) {
		syslog(LOG_WARNING, "pipe failed");
		return -1;
	}


	pid = fork();

	switch(pid) {
	case 0:
		/* child */
		close(STDOUT_FILENO);

		/* dup write side of pipe to STDOUT */
		dup(fds[1]);

		syslog(LOG_DEBUG, "execute '%s' with given arguments", argv[0]);
		retval = execv(argv[0], argv);

		syslog(LOG_DEBUG, FIELD_PROGRAM " execution of '%s' failed",
		       path);
		exit(-1);
	case -1:
		syslog(LOG_WARNING, "fork failed");
		return -1;
	default:
		/* parent reads from fds[0] */
		close(fds[1]);
		retval = 0;
		i = 0;
		while (1) {
			count = read(fds[0], value + i, len - i-1);
			if (count <= 0)
				break;

			i += count;
			if (i >= len-1) {
				syslog(LOG_WARNING, "result len %d too short",
				       len);
				retval = -1;
				break;
			}
		}

		if (count < 0) {
			syslog(LOG_WARNING, "read failed with '%s'",
			       strerror(errno));
			retval = -1;
		}

		if (i > 0 && value[i-1] == '\n')
			i--;
		value[i] = '\0';
		syslog(LOG_DEBUG, "result is '%s'", value);

		close(fds[0]);
		wait(&status);

		if (!WIFEXITED(status) || (WEXITSTATUS(status) != 0)) {
			syslog(LOG_DEBUG, "exec program status 0x%x", status);
			retval = -1;
		}
	}
	return retval;
}
